package  
{
	import flash.display.BitmapData;
	import net.flashpunk.graphics.Graphiclist;
	import net.flashpunk.graphics.Image;
	import net.flashpunk.FP;
	import net.flashpunk.graphics.Stamp;
	import net.flashpunk.graphics.Text;
	import net.flashpunk.Sfx;
	import net.flashpunk.tweens.misc.Alarm;
	import net.flashpunk.tweens.misc.VarTween;
	import net.flashpunk.tweens.sound.SfxFader;
	import net.flashpunk.utils.Input;
	
	/**
	 * Asteroid game object
	 * 
	 * @author Switchbreak
	 */
	public class Asteroid extends CelestialBody 
	{
		// Embeds
		[Embed(source = "../img/Asteroid.png")] private static const ASTEROID:Class;
		[Embed(source = "../img/SelfDestruct.png")] private static const SELF_DESTRUCT:Class;
		[Embed(source = "../snd/Hit_Hurt3.mp3")] private static const HURT:Class;
		[Embed(source = "../snd/Explosion.mp3")] private static const EXPLODE:Class;
		[Embed(source = "../snd/Thruster.mp3")] private static const THRUSTER:Class;
		[Embed(source = "../snd/rain.mp3")] private static const RAIN:Class;
		
		// Constants
		static private const RADIUS:Number = 10;
		static private const DISTANCE:Number = 150;
		static private const THROW_POWER:Number = 10;
		static private const AFTERTOUCH_POWER:Number = 20;
		static private const MAX_AFTERTOUCH:Number = 500;
		static private const RANGE:Number = 7000;
		static private const SPAWN_DURATION:Number = 1;
		static private const RAIN_THRESHOLD:Number = 250;
		static private const RAIN_RATE:Number = 10;
		static private const MAX_BACTERIA:Number = 100;
		
		static private const COLOR:uint = 0xFFFF00;
		static private const PADDING:int = 10;
		static private const LIFE_BAR_THICKNESS:int = 20;
		static private const LIFE_BAR_BORDER_THICKNESS:int = 5;
		
		static private const TAIL_PARTICLE_SIZE:int = 10;
		static private const TAIL_PARTICLE_COLOR:uint = 0xFFFF00;
		static private const TAIL_PARTICLE_DURATION:Number = 2;
		static private const THRUST_PARTICLE_SIZE:int = 5;
		static private const THRUST_PARTICLE_COLOR:uint = 0xFFFFFF;
		static private const THRUST_PARTICLE_DURATION:Number = 0.5;
		static private const EXPLODE_PARTICLE_COUNT:int = 10;
		static private const EXPLODE_PARTICLE_SPEED:int = 25;
		static private const EXPLODE_PARTICLE_SIZE:int = 15;
		static private const EXPLODE_PARTICLE_COLOR:uint = 0xFFFF00;
		static private const EXPLODE_PARTICLE_DURATION:Number = 1;
		static private const RAIN_PARTICLE_SIZE:int = 5;
		static private const RAIN_PARTICLE_SPEED:int = 200;
		static private const RAIN_PARTICLE_COLOR:uint = 0x00FF00;
		static private const RAIN_PARTICLE_DURATION:Number = 1;
		
		// State enumeration
		private static const STATE_CIRCLING:int = 0;
		private static const STATE_FLYING:int = 1;
		private static const STATE_SPAWNING:int = 2;
		private static const STATE_EXPLODING:int = 3;
		static private const SOUND_EFFECTS_VOLUME:Number = 0.5;
		
		// Variables
		public var asteroidState:int = STATE_CIRCLING;
		public var bacteriaLevel:Number = 0;
		private var raining:Boolean = false;
		
		// Graphics
		private var asteroidSprite:Image;
		private var particleEmitter:Graphiclist;
		private var thrusterEmitter:Graphiclist;
		private var lifeBar:LifeBar;
		private var destructButton:Stamp;
		
		// Objects
		private var thruster:Sfx;
		private var rain:Sfx;
		
		/**
		 * Create and initialize the asteroid
		 * 
		 * @param	percent
		 * Orbital position of the asteroid as a percantage of a full 360 degree
		 * circle
		 */
		public function Asteroid( percent:Number, spawnState:Boolean = false ) 
		{
			asteroidSprite = new Image( ASTEROID );//Image.createCircle( RADIUS, COLOR );
			asteroidSprite.smooth = true;
			
			super( DISTANCE, RADIUS, asteroidSprite, 2 * Math.PI * percent );
			collidable = false;
			
			thruster = new Sfx( THRUSTER );
			rain = new Sfx( RAIN );
			rain.loop( 0 );
			
			// If this meteor is being created in spawn state, make it
			// semi-transparent and uncollidable until after a set time
			if ( spawnState )
			{
				asteroidSprite.alpha = 0;
				
				var alphaTween:VarTween = new VarTween( SpawnIn, ONESHOT );
				alphaTween.tween( asteroidSprite, "alpha", 1, SPAWN_DURATION );
				addTween( alphaTween, true );
			}
		}
		
		/**
		 * Handle collision detection
		 */
		override public function update():void 
		{
			if ( asteroidState == STATE_FLYING )
			{
				// Check for flying out of range
				if ( xPosition < -RANGE || xPosition > RANGE || yPosition < -RANGE || yPosition > RANGE )
				{
					Explode();
				}
				
				// Check for collision against all other celestial bodies
				var bodies:Vector.<CelestialBody> = new Vector.<CelestialBody>();
				world.getClass( CelestialBody, bodies );
				
				raining = false;
				for each( var body:CelestialBody in bodies )
				{
					if ( body != this && body.collidable )
					{
						// Check square of the distance vs. square of summed radii
						var distanceSq:Number = (body.xPosition - xPosition) * (body.xPosition - xPosition) + (body.yPosition - yPosition) * (body.yPosition - yPosition);
						if ( distanceSq	<= (radius + body.radius) * (radius + body.radius) )
						{
							body.CollisionResponse( this );
							CollisionResponse( body );
							break;
						}
						else if ( body is Planet && bacteriaLevel > 0 &&
								distanceSq <= (body.radius + Math.max( body.radius, RAIN_THRESHOLD ) ) * (body.radius + Math.max( body.radius, RAIN_THRESHOLD ) ) )
						{
							Rain( Planet( body ) );
						}
					}
				}
				
				// Check for collision against UFOs
				var ufos:Vector.<UFO> = new Vector.<UFO>();
				world.getClass( UFO, ufos );
				for each( var ufo:UFO in ufos )
				{
					if ( ufo.collidable )
					{
						if ( FP.distanceRectPoint( xPosition, yPosition, ufo.xPosition - ufo.halfWidth, ufo.yPosition - ufo.halfHeight, ufo.width, ufo.height ) < radius )
						{
							ufo.Explode();
							var sound:Sfx = new Sfx( EXPLODE );
							sound.play( SOUND_EFFECTS_VOLUME );
							GameWorld.score += 1000;
						}
					}
				}
				
				if ( raining && rain.volume == 0 )
				{
					var sfxTween:SfxFader = new SfxFader( rain, null, ONESHOT );
					sfxTween.fadeTo( 0.1, 1 );
					addTween( sfxTween, true );
				}
				else if ( !raining && rain.volume > 0 )
				{
					sfxTween = new SfxFader( rain, null, ONESHOT );
					sfxTween.fadeTo( 0, 1 );
					addTween( sfxTween, true );
				}
				
				// Handle input for aftertouch
				if ( Input.mouseDown )
				{
					var xVector:Number = (world.mouseX - x);
					var yVector:Number = (world.mouseY - y);
					
					// Cap movement vector magnitude
					var magnitude:Number = MAX_AFTERTOUCH / Math.sqrt( xVector * xVector + yVector * yVector );
					xVector *= magnitude;
					yVector *= magnitude;
					
					xVelocity -= xVector * FP.elapsed;
					yVelocity -= yVector * FP.elapsed;
					
					ThrustParticle( xVector, yVector );
					if ( !thruster.playing )
						thruster.loop( 0.01 );
				}
				else
				{
					if ( thruster.playing )
						thruster.stop();
				}
				
				// Handle input for destruct button
				if ( Input.mousePressed && Input.mouseX >= destructButton.x && Input.mouseX <= destructButton.x + destructButton.width &&
					Input.mouseY >= destructButton.y && Input.mouseY <= destructButton.y + destructButton.height )
				{
					Explode();
				}
				
				TailParticle();
			}
			
			super.update();
		}
		
		override public function CollisionResponse( body:CelestialBody ):void 
		{
			GameWorld.multiplier = 1;
			
			if ( body is Planet )
			{
				var sound:Sfx = new Sfx( HURT );
				sound.play( SOUND_EFFECTS_VOLUME );
				
				bacteriaLevel += Planet( body ).Gather();
				if ( bacteriaLevel > MAX_BACTERIA )
					bacteriaLevel = MAX_BACTERIA;
				
				lifeBar.SetLife( bacteriaLevel );
				
				// Get relationship of asteroid and planet
				var distance:Number = Math.sqrt( (body.xPosition - xPosition) * (body.xPosition - xPosition) + (body.yPosition - yPosition) * (body.yPosition - yPosition) );
				var xVector:Number = (body.xPosition - xPosition) / distance;
				var yVector:Number = (body.yPosition - yPosition) / distance;
				
				// Resolve intersection
				var overlap:Number = (radius + body.radius) - distance;
				
				xPosition -= xVector * overlap;
				yPosition -= yVector * overlap;
				
				// Bounce asteroid off of planet
				var d1:Number = xVelocity * xVector + yVelocity * yVector;
				var d2:Number = body.xVelocity * xVector + body.yVelocity * yVector;
				var p:Number = (2 * (d1 - d2));
				
				xVelocity -= p * xVector * 1.1;
				yVelocity -= p * yVector * 1.1;
			}
			else
			{
				Explode();
			}
		}
		
		override public function removed():void 
		{
			thruster.stop();
			rain.stop();
			
			super.removed();
		}
		
		private function Rain( planet:Planet ):void
		{
			if ( bacteriaLevel <= 0 || planet.health < Planet.MAX_HEALTH || planet.evolution >= Planet.MAX_EVOLUTION )
				return;
			
			raining = true;
			
			// Calculate rain amount based on rate, capped at available level
			var rainAmount:Number = RAIN_RATE * FP.elapsed;
			if ( rainAmount > bacteriaLevel )
				rainAmount = bacteriaLevel;
			
			GameWorld.score += rainAmount * Math.floor( GameWorld.multiplier );
			GameWorld.multiplier += rainAmount / 10;
			
			// Reduce current amount and update bar
			bacteriaLevel -= rainAmount;
			lifeBar.SetLife( bacteriaLevel );
			
			// Show rain particle effect
			var xVector:Number = planet.xPosition - xPosition;
			var yVector:Number = planet.yPosition - yPosition;
			var magnitude:Number = RAIN_PARTICLE_SPEED / Math.sqrt( xVector * xVector + yVector * yVector );
			xVector *= magnitude;
			yVector *= magnitude;
			particleEmitter.add( new Particle( xPosition, yPosition, RAIN_PARTICLE_SIZE, RAIN_PARTICLE_COLOR, xVector, yVector, RAIN_PARTICLE_DURATION, RemoveParticle, planet, RAIN_PARTICLE_SPEED ) );
			
			// Send rain to planet
			planet.Rain( rainAmount );
		}
		
		/**
		 * Throw the asteroid
		 * 
		 * @param	xThrow
		 * Strength and direction of the throw
		 * @param	yThrow
		 * Strength and direction of the throw
		 */
		public function Throw( xThrow:Number, yThrow:Number ):void
		{
			// Replace the asteroid with a newly spawned one
			ReplaceAsteroid();
			
			// Impart velocity instantly to the asteroid
			xVelocity = (xVelocity / 2) - xThrow * THROW_POWER;
			yVelocity = (yVelocity / 2) - yThrow * THROW_POWER;
			affectedByPlanets = true;
			layer = -1;
			asteroidState = STATE_FLYING;
			
			// Create thruster particle effect
			particleEmitter = new Graphiclist();
			particleEmitter.active = true;
			addGraphic( particleEmitter );
			
			// Create thruster particle effect
			particleEmitter = new Graphiclist();
			particleEmitter.active = true;
			addGraphic( particleEmitter );
			
			// Create life bar UI element
			lifeBar = new LifeBar( PADDING, FP.height - LIFE_BAR_THICKNESS - PADDING, FP.width - (PADDING << 1), LIFE_BAR_THICKNESS, LIFE_BAR_BORDER_THICKNESS, "Bacteria" );
			lifeBar.SetLife( bacteriaLevel );
			lifeBar.relative = false;
			lifeBar.scrollX = 0;
			lifeBar.scrollY = 0;
			addGraphic( lifeBar );
			
			// Create self-destruct button UI element
			destructButton = new Stamp( SELF_DESTRUCT ); //Image.createRect( 20, 20, 0xFF0000 );
			destructButton.relative = false;
			destructButton.scrollX = 0;
			destructButton.scrollY = 0;
			destructButton.x = FP.width - destructButton.width - PADDING;
			destructButton.y = PADDING;
			addGraphic( destructButton );
		}
		
		/**
		 * Emit a new particle for the asteroid trail
		 */
		private function TailParticle():void
		{
			particleEmitter.add( new Particle( xPosition, yPosition, TAIL_PARTICLE_SIZE, TAIL_PARTICLE_COLOR, 0, 0, TAIL_PARTICLE_DURATION, RemoveParticle ) );
		}
		
		/**
		 * Emit a new particle for the asteroid aftertouch
		 */
		private function ThrustParticle( xVector:Number, yVector:Number ):void
		{
			particleEmitter.add( new Particle( xPosition, yPosition, THRUST_PARTICLE_SIZE, THRUST_PARTICLE_COLOR, xVector, yVector, THRUST_PARTICLE_DURATION, RemoveParticle ) );
		}
		
		/**
		 * Remove a previously emitted particle
		 * 
		 * @param	particle
		 * Particle to remove
		 */
		private function RemoveParticle( particle:Particle ):void
		{
			particleEmitter.remove( particle );
		}
		
		/**
		 * Leave spawn state and become collidable
		 */
		private function SpawnIn():void
		{
			asteroidState = STATE_CIRCLING;
			asteroidSprite.alpha = 1;
			//collidable = true;
		}
		
		/**
		 * Replace the asteroid with a new one in the same position
		 */
		public function ReplaceAsteroid():void
		{
			// Spawn replacement asteroid in same spot
			var replacement:Asteroid = new Asteroid( 0, true );
			
			replacement.xPosition = xPosition;
			replacement.yPosition = yPosition;
			replacement.xVelocity = xVelocity;
			replacement.yVelocity = yVelocity;
			replacement.xAcceleration = xAcceleration;
			replacement.yAcceleration = yAcceleration;
			
			world.add( replacement );
		}
		
		/**
		 * Explode the asteroid
		 */
		public function Explode():void
		{
			var sound:Sfx = new Sfx( EXPLODE );
			sound.play( SOUND_EFFECTS_VOLUME );
			
			if ( asteroidState == STATE_CIRCLING )
				ReplaceAsteroid();
			
			asteroidState = STATE_EXPLODING;
			collidable = false;
			moving = false;
			asteroidSprite.visible = false;
			
			if ( particleEmitter == null )
			{
				particleEmitter = new Graphiclist();
				particleEmitter.active = true;
				addGraphic( particleEmitter );
			}
			
			for ( var i:int; i < EXPLODE_PARTICLE_COUNT; ++i )
			{
				var xVector:Number = FP.random * EXPLODE_PARTICLE_SPEED * 2 - EXPLODE_PARTICLE_SPEED;
				var yVector:Number = FP.random * EXPLODE_PARTICLE_SPEED * 2 - EXPLODE_PARTICLE_SPEED;
				particleEmitter.add( new Particle( xPosition, yPosition, EXPLODE_PARTICLE_SIZE, EXPLODE_PARTICLE_COLOR, xVector, yVector, EXPLODE_PARTICLE_DURATION, RemoveParticle ) );
			}
			
			addTween( new Alarm( EXPLODE_PARTICLE_DURATION, RemoveSelf, ONESHOT ), true );
		}
		
		public function RemoveSelf():void
		{
			(world as GameWorld).RemoveAsteroid( this );
		}
		
	}

}