package  
{
	import flash.geom.Point;
	import flash.display.BitmapData;
	import net.flashpunk.graphics.Image;
	import net.flashpunk.FP;
	
	/**
	 * Asteroid particle graphic element
	 * 
	 * @author Switchbreak
	 */
	public class Particle extends Image 
	{
		
		// Constants
		static private const PARTICLE_DURATION_RANGE:Number = 0.1;
		static private const POSITION_RANGE:Number = 10;
		static private const THRUSTER_SPEED:Number = 1;
		
		// Private variables
		private var xPosition:Number;
		private var yPosition:Number;
		private var xVelocity:Number;
		private var yVelocity:Number;
		private var timer:Number;
		private var alphaRate:Number;
		private var callback:Function;
		private var target:Object;
		private var speed:Number;
		
		/**
		 * Create and initialize the particle
		 * 
		 * @param	setX
		 * Position of the particle
		 * @param	setY
		 * Position of the particle
		 */
		public function Particle( setX:Number, setY:Number, size:int, color:uint, xVector:Number, yVector:Number, duration:Number, setCallback:Function, setTarget:Object = null, setSpeed:Number = 0 ) 
		{
			timer = duration + FP.random * (PARTICLE_DURATION_RANGE << 1) - PARTICLE_DURATION_RANGE;
			alphaRate = 1 / timer;
			xPosition = setX - (size >> 1) + FP.random * (POSITION_RANGE << 1) - POSITION_RANGE;
			yPosition = setY - (size >> 1) + FP.random * (POSITION_RANGE << 1) - POSITION_RANGE;
			xVelocity = xVector * THRUSTER_SPEED;
			yVelocity = yVector * THRUSTER_SPEED;
			callback = setCallback;
			target = setTarget;
			speed = setSpeed * THRUSTER_SPEED;
			
			var particleSource:BitmapData = new BitmapData( size, size, false, color );
			super( particleSource );
			
			active = true;
			relative = false;
			x = xPosition * GameWorld.zoom;
			y = yPosition * GameWorld.zoom;
			scale = GameWorld.zoom;
		}
		
		/**
		 * Fade out over time
		 */
		override public function update():void 
		{
			if ( target != null )
			{
				xVelocity = target.xPosition - xPosition;
				yVelocity = target.yPosition - yPosition;
				
				var magnitude:Number = speed / Math.sqrt( xVelocity * xVelocity + yVelocity * yVelocity );
				xVelocity *= magnitude;
				yVelocity *= magnitude;
			}
			
			xPosition += xVelocity * FP.elapsed;
			yPosition += yVelocity * FP.elapsed;
			
			// Fade particle out over DURATION
			timer -= FP.elapsed;
			alpha -= FP.elapsed * alphaRate;
			if ( timer <= 0 )
			{
				callback( this );
			}
			
			// Handle scaling for zoom
			x = xPosition * GameWorld.zoom;
			y = yPosition * GameWorld.zoom;
			scale = GameWorld.zoom;
			
			super.update();
		}
	}

}