package  
{
	import adobe.utils.CustomActions;
	import net.flashpunk.World;
	import net.flashpunk.graphics.Stamp;
	import net.flashpunk.graphics.Text;
	import net.flashpunk.graphics.TiledImage;
	import net.flashpunk.tweens.misc.VarTween;
	import net.flashpunk.World;
	import net.flashpunk.FP;
	import net.flashpunk.utils.*;
	
	/**
	 * Start Screen
	 * 
	 * @author Switchbreak
	 */
	public class StartScreen extends World 
	{
		[Embed(source = "../img/Nebula.png")] private static const NEBULA_BG:Class;
		
		private var startButton:Text;
		private var tutorialButton:Text;
		
		private var dialogShowing:Boolean = false;
		
		public function StartScreen() 
		{
			// Set game font
			Text.font = "UI Font";
			
			// Center the scene on origin 0, 0
			camera.x = -FP.halfWidth;
			camera.y = -FP.halfHeight;
			
			// Create game objects
			var sun:Sun = new Sun();
			add( sun );
			
			add( new Asteroid( 60 ) );
			
			add( new Planet( 400, 40, 0x00FF77, 100, true ) );
			add( new Planet( 700, 60, 0x00FF77, 0, true ) );
			add( new Planet( 1000, 80, 0x00FF77, 0, true ) );
			add( new Planet( 1400, 100, 0x00FF77, 0, true ) );
			add( new Planet( 1800, 100, 0x00FF77, 0, true ) );
			add( new Planet( 2500, 200, 0x00FF77, 0, true ) );
			add( new Planet( 3000, 200, 0x00FF77, 0, true ) );
			add( new Planet( 3500, 200, 0x00FF77, 0, true ) );
			
			addGraphic( new Starfield(), 1000 );
			
			var nebula:TiledImage = new TiledImage( NEBULA_BG, 1280, 960 );
			nebula.scrollX = 0;
			nebula.scrollY = 0;
			nebula.setOffset( -(camera.x + FP.halfWidth) / GameWorld.zoom, -(camera.y + FP.halfHeight) / GameWorld.zoom );
			addGraphic( nebula, 1000 );
			nebula.scale = GameWorld.zoom * 10;
			nebula.x = FP.halfWidth - 6400 * GameWorld.zoom;
			nebula.y = FP.halfHeight - 4800 * GameWorld.zoom;
			
			var titleText:Text = new Text( "SUN GOD" );
			titleText.size = 100;
			titleText.y = (FP.height >> 2) - (titleText.height >> 1);
			titleText.x = 0;
			titleText.width = FP.width;
			titleText.align = "center";
			titleText.scrollX = 0;
			titleText.scrollY = 0;
			addGraphic( titleText );
			
			startButton = new Text( "Start Game" );
			startButton.size = 36;
			startButton.y = FP.halfHeight + (FP.height >> 2) - (startButton.height >> 1);
			startButton.x = FP.halfWidth - (startButton.width >> 1);
			startButton.align = "center";
			startButton.scrollX = 0;
			startButton.scrollY = 0;
			addGraphic( startButton );
			
			tutorialButton = new Text( "How To Play" );
			tutorialButton.size = 36;
			tutorialButton.y = startButton.y + startButton.height;
			tutorialButton.x = FP.halfWidth - (tutorialButton.width >> 1);
			tutorialButton.align = "center";
			tutorialButton.scrollX = 0;
			tutorialButton.scrollY = 0;
			addGraphic( tutorialButton );
		}
		
		override public function update():void 
		{
			if ( !dialogShowing && Input.mousePressed )
			{
				if( Input.mouseX >= startButton.x && Input.mouseX <= startButton.x + startButton.width &&
					Input.mouseY >= startButton.y && Input.mouseY <= startButton.y + startButton.height )
				{
					FP.world = new GameWorld();
				}
				else if( Input.mouseX >= tutorialButton.x && Input.mouseX <= tutorialButton.x + tutorialButton.width &&
					Input.mouseY >= tutorialButton.y && Input.mouseY <= tutorialButton.y + tutorialButton.height )
				{
					add( new TutorialScreen( HideTutorial ) );
					dialogShowing = true;
				}
			}
			
			super.update();
		}
		
		public function HideTutorial( tutorial:TutorialScreen ):void
		{
			remove( tutorial );
			dialogShowing = false;
		}
	}

}